
require 'multi_json'
require 'base64'


def appborg_decodeLine(line)
  MultiJson.decode Base64::decode64 line
end

def appborg_encodeLine(x)
  json = MultiJson.encode x
  json64 = Base64::strict_encode64 json
  json64 + "\n"
end


module Appborg

  DEFAULT_WRAPPER_NAME = 'subprocess'
  DEFAULT_ROUTER_NAME = 'router'

  class PTYWrapper
    def initialize(output, input, pid)
      @output = output
      @input = input
      @pid = pid
    end

    def killDashNine()
      # "Kill dash nine / no more CPU time!"
      `kill -p #{@pid}`
    end

    def sendEventSync(e)
      @input.write appborg_encodeLine e
      buffer = ""
      2.times do
        buffer = ""
        @output.readpartial(250000, buffer) until buffer =~ /\n/
      end
      appborg_decodeLine buffer
    end
  end


  class Router
    def initialize()
      @names = [DEFAULT_ROUTER_NAME]
      @wrappers = {}
      @counter = 0
    end

    def _nextId()
      @counter += 1
      @counter
    end

    def addPTY(output, input, pid, name=DEFAULT_WRAPPER_NAME)
      @wrappers[name] = PTYWrapper.new(output, input, pid)
    end

    def sendSync(method, info={})
      dest = DEFAULT_WRAPPER_NAME
      request = {
        :id       => _nextId(),
        :from     => @names[0],
        :to       => dest,
        :method   => method,
        :info     => info
      }
      response = @wrappers[dest].sendEventSync request
      if not response['error'].nil?
        throw response['error']
      else
        return response['result']
      end
    end

    def killDashNine()
      @wrappers.each do |k, wrapper|
        wrapper.killDashNine()
      end
    end
  end
end
